@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,8 @@
  * @brief Class @ref Corrade::Containers::Array
  */
 
+#include <initializer_list>
+#include <new>
 #include <type_traits>
 #include <utility>
 
@@ -36,10 +38,6 @@
 #include "Corrade/Containers/ArrayView.h"
 #include "Corrade/Containers/Tags.h"
 
-#ifdef CORRADE_BUILD_DEPRECATED
-#include "Corrade/Utility/Macros.h"
-#endif
-
 namespace Corrade { namespace Containers {
 
 namespace Implementation {
@@ -85,16 +83,7 @@ additionally also in range-for cycle.
 
 Usage example:
 
-@code{.cpp}
-// Create default-initialized array with 5 integers and set them to some value
-Containers::Array<int> a{5};
-int b = 0;
-for(auto& i: a) i = b++; // a = {0, 1, 2, 3, 4}
-
-// Create array from given values
-auto b = Containers::Array<int>::from(3, 18, -157, 0);
-b[3] = 25; // b = {3, 18, -157, 25}
-@endcode
+@snippet Containers.cpp Array-usage
 
 @section Containers-Array-initialization Array initialization
 
@@ -103,39 +92,24 @@ are not initialized at all and default constructor is called on other types. It
 is possible to initialize the array in a different way using so-called *tags*:
 
 -   @ref Array(DefaultInitT, std::size_t) is equivalent to the default case
-    (useful when you want to make the choice appear explicit).
+    (useful when you want to make the choice appear explicit). In other words,
+    @cpp new T[size] @ce.
 -   @ref Array(ValueInitT, std::size_t) zero-initializes trivial types and
-    calls default constructor elsewhere.
+    calls default constructor elsewhere. In other words,
+    @cpp new T[size]{} @ce.
 -   @ref Array(DirectInitT, std::size_t, Args&&... args) constructs all
-    elements of the array using provided arguments.
+    elements of the array using provided arguments. In other words,
+    @cpp new T[size]{T{args...}, T{args...}, …} @ce.
+-   @ref Array(InPlaceInitT, std::initializer_list<T>) allocates unitialized
+    memory and then copy-constructs all elements from the initializer list. In
+    other words, @cpp new T[size]{args...} @ce.
 -   @ref Array(NoInitT, std::size_t) does not initialize anything and you need
     to call the constructor on all elements manually using placement new,
     @ref std::uninitialized_copy() or similar. This is the dangerous option.
 
 Example:
 
-@code{.cpp}
-// These are equivalent
-Containers::Array<int> a1{5};
-Containers::Array<int> a2{Containers::DefaultInit, 5};
-
-// Array of 100 zeros
-Containers::Array<int> b{Containers::ValueInit, 100};
-
-// Array of type with no default constructor
-struct Vec3 {
-    Vec3(float, float, float);
-};
-Containers::Array<Vec3> c{Containers::DirectInit, 5, 5.2f, 0.4f, 1.0f};
-
-// Manual construction of each element
-struct Foo {
-    Foo(int index);
-};
-Containers::Array<Foo> d{Containers::NoInit, 5};
-int index = 0;
-for(Foo& f: d) new(&f) Foo(index++);
-@endcode
+@snippet Containers.cpp Array-initialization
 
 @section Containers-Array-wrapping Wrapping externally allocated arrays
 
@@ -150,38 +124,56 @@ contents using @cpp operator delete[] @ce.
 
 For example, properly deallocating array allocated using @ref std::malloc():
 
-@code{.cpp}
-const int* data = reinterpret_cast<int*>(std::malloc(25*sizeof(int)));
-
-// Will call std::free() on destruction
-Containers::Array<int> array{data, 25, [](int* data, std::size_t) { std::free(data); }};
-@endcode
+@snippet Containers.cpp Array-wrapping
 
 By default, plain function pointers are used to avoid having the type affected
 by the deleter function. If the deleter needs to manage some state, a custom
 deleter type can be used:
 
-@code{.cpp}
-struct UnmapBuffer {
-    UnmapBuffer(GLuint id): _id{id} {}
-    void operator()(T*, std::size_t) { glUnmapNamedBuffer(_id); }
+@snippet Containers.cpp Array-deleter
 
-private:
-    GLuint _id;
-};
+@section Containers-Array-views Conversion to array views
 
-GLuint buffer;
-char* data = reinterpret_cast<char*>(glMapNamedBuffer(buffer, GL_READ_WRITE));
+Arrays are implicitly convertible to @ref ArrayView as described in the
+following table. The conversion is only allowed if @cpp T* @ce is implicitly
+convertible to @cpp U* @ce (or both are the same type) and both have the same
+size. This also extends to other container types constructibe from
+@ref ArrayView, which means for example that a @ref StridedArrayView1D is
+implicitly convertible from @ref Array as well.
 
-// Will unmap the buffer on destruction
-Containers::Array<char, UnmapBuffer> array{data, bufferSize, UnmapBuffer{buffer}};
-@endcode
+Owning array type               | ↭ | Non-owning view type
+------------------------------- | - | ---------------------
+@ref Array "Array<T>"           | → | @ref ArrayView "ArrayView<U>"
+@ref Array "Array<T>"           | → | @ref ArrayView "ArrayView<const U>"
+@ref Array "const Array<T>"     | → | @ref ArrayView "ArrayView<const U>"
 
-@see @ref arrayCast(Array<T, D>&)
+@section Containers-Array-stl STL compatibility
 
-@todo Something like ArrayTuple to create more than one array with single
-    allocation and proper alignment for each type? How would non-POD types be
-    constructed in that? Will that be useful in more than one place?
+On compilers that support C++2a and @ref std::span, implicit conversion of an
+@ref Array to it is provided in @ref Corrade/Containers/ArrayViewStlSpan.h. The
+conversion is provided in a separate header to avoid unconditional
+@cpp #include <span> @ce, which significantly affects compile times. The
+following table lists allowed conversions:
+
+Corrade type                    | ↭ | STL type
+------------------------------- | - | ---------------------
+@ref Array "Array<T>"           | → | @ref std::span "std::span<T>"
+@ref Array "Array<T>"           | → | @ref std::span "std::span<const T>"
+@ref Array "const Array<T>"     | → | @ref std::span "std::span<const T>"
+
+There are some dangerous corner cases due to the way @ref std::span is
+designed, see @ref Containers-ArrayView-stl "ArrayView STL compatibility" for
+more information.
+
+@m_class{m-block m-success}
+
+@par Single-header version
+    This class, together with @ref StaticArray, is also available as a
+    single-header [CorradeArray.h](https://github.com/mosra/magnum-singles/tree/master/CorradeArray.h)
+    library in the Magnum Singles repository for easier integration into your
+    projects. See @ref corrade-singles for more information.
+
+@see @ref arrayCast(Array<T, D>&), @ref StaticArray
 */
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T, class D = void(*)(T*, std::size_t)>
@@ -193,24 +185,6 @@ class Array {
         typedef T Type;     /**< @brief Element type */
         typedef D Deleter;  /**< @brief Deleter type */
 
-        #ifdef CORRADE_BUILD_DEPRECATED
-        /**
-         * @brief @copybrief Array(InPlaceInitT, std::initializer_list<T>)
-         * @deprecated Use @ref Array(InPlaceInitT, std::initializer_list<T>) instead.
-         */
-        template<class ...U> CORRADE_DEPRECATED("use Array(InPlaceInitT, std::initializer_list<T>) instead") static Array<T, D> from(U&&... values) {
-            return Array<T, D>{InPlaceInit, {T(std::forward<U>(values))...}};
-        }
-
-        /**
-         * @brief @copybrief Array(ValueInitT, std::size_t)
-         * @deprecated Use @ref Array(ValueInitT, std::size_t) instead.
-         */
-        CORRADE_DEPRECATED("use Array(ValueInitT, std::size_t) instead") static Array<T, D> zeroInitialized(std::size_t size) {
-            return Array<T>{ValueInit, size};
-        }
-        #endif
-
         /** @brief Conversion from nullptr */
         #ifdef DOXYGEN_GENERATING_OUTPUT
         /*implicit*/ Array(std::nullptr_t) noexcept:
@@ -324,71 +298,43 @@ class Array {
         /** @brief Move assignment */
         Array<T, D>& operator=(Array<T, D>&&) noexcept;
 
-        #ifndef CORRADE_MSVC2017_COMPATIBILITY
-        /** @brief Whether the array is non-empty */
-        /* Disabled on MSVC <= 2017 to avoid ambiguous operator+() when doing
-           pointer arithmetic. */
-        explicit operator bool() const { return _data; }
-        #endif
+        /* The following view conversion is *not* restricted to this& because
+           that would break uses like `consume(foo());`, where `consume()`
+           expects a view but `foo()` returns an owning array. */
 
         /**
-         * @brief Convert to @ref ArrayView
+         * @brief Convert to external view representation
          *
-         * Enabled only if @cpp T* @ce is implicitly convertible to @cpp U* @ce.
-         * Expects that both types have the same size.
-         * @see @ref arrayView(Array<T, D>&)
+         * @see @ref Containers-Array-stl
          */
-        #ifdef DOXYGEN_GENERATING_OUTPUT
-        template<class U>
-        #else
-        template<class U, class = typename std::enable_if<!std::is_void<U>::value && std::is_convertible<T*, U*>::value>::type>
-        #endif
-        /*implicit*/ operator ArrayView<U>() noexcept {
-            static_assert(sizeof(T) == sizeof(U), "type sizes are not compatible");
-            return {_data, _size};
-        }
-
-        /**
-         * @brief Convert to const @ref ArrayView
-         *
-         * Enabled only if @cpp T* @ce or @cpp const T* @ce is implicitly
-         * convertible to @cpp U* @ce. Expects that both types have the same
-         * size.
-         * @see @ref arrayView(const Array<T, D>&)
-         */
-        #ifdef DOXYGEN_GENERATING_OUTPUT
-        template<class U>
-        #else
-        template<class U, class = typename std::enable_if<std::is_convertible<T*, U*>::value || std::is_convertible<T*, const U*>::value>::type>
-        #endif
-        /*implicit*/ operator ArrayView<const U>() const noexcept {
-            static_assert(sizeof(T) == sizeof(U), "type sizes are not compatible");
-            return {_data, _size};
+        template<class U, class = decltype(Implementation::ArrayViewConverter<T, U>::to(std::declval<ArrayView<T>>()))> /*implicit*/ operator U() {
+            return Implementation::ArrayViewConverter<T, U>::to(*this);
         }
 
         /** @overload */
-        /*implicit*/ operator ArrayView<const void>() const noexcept {
-            /* Yes, the size is properly multiplied by sizeof(T) by the constructor */
-            return {_data, _size};
+        template<class U, class = decltype(Implementation::ArrayViewConverter<const T, U>::to(std::declval<ArrayView<const T>>()))> constexpr /*implicit*/ operator U() const {
+            return Implementation::ArrayViewConverter<const T, U>::to(*this);
         }
 
+        #ifndef CORRADE_MSVC2019_COMPATIBILITY
+        /** @brief Whether the array is non-empty */
+        /* Disabled on MSVC <= 2017 to avoid ambiguous operator+() when doing
+           pointer arithmetic. On MSVC 2019 this works properly when
+           /permissive- is enabled, but I can neither detect a presence of that
+           flag nor force it for all users (since it often ICEs and breaks 3rd
+           party code), so disabling for 2019 as well. */
+        explicit operator bool() const { return _data; }
+        #endif
+
         /* `char* a = Containers::Array<char>(5); a[3] = 5;` would result in
            instant segfault, disallowing it in the following conversion
            operators */
 
         /** @brief Conversion to array type */
-        /*implicit*/ operator T*()
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        &
-        #endif
-        { return _data; }
+        /*implicit*/ operator T*() & { return _data; }
 
         /** @overload */
-        /*implicit*/ operator const T*() const
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        &
-        #endif
-        { return _data; }
+        /*implicit*/ operator const T*() const & { return _data; }
 
         /** @brief Array data */
         T* data() { return _data; }
@@ -409,16 +355,42 @@ class Array {
         /** @brief Whether the array is empty */
         bool empty() const { return !_size; }
 
-        /** @brief Pointer to first element */
+        /**
+         * @brief Pointer to first element
+         *
+         * @see @ref front()
+         */
         T* begin() { return _data; }
         const T* begin() const { return _data; }        /**< @overload */
         const T* cbegin() const { return _data; }       /**< @overload */
 
-        /** @brief Pointer to (one item after) last element */
+        /**
+         * @brief Pointer to (one item after) last element
+         *
+         * @see @ref back()
+         */
         T* end() { return _data+_size; }
         const T* end() const { return _data+_size; }    /**< @overload */
         const T* cend() const { return _data+_size; }   /**< @overload */
 
+        /**
+         * @brief First element
+         *
+         * Expects there is at least one element.
+         * @see @ref begin()
+         */
+        T& front();
+        const T& front() const; /**< @overload */
+
+        /**
+         * @brief Last element
+         *
+         * Expects there is at least one element.
+         * @see @ref end()
+         */
+        T& back();
+        const T& back() const; /**< @overload */
+
         /**
          * @brief Reference to array slice
          *
@@ -433,11 +405,11 @@ class Array {
         }
         /** @overload */
         ArrayView<T> slice(std::size_t begin, std::size_t end) {
-            return slice(_data + begin, _data + end);
+            return ArrayView<T>(*this).slice(begin, end);
         }
         /** @overload */
         ArrayView<const T> slice(std::size_t begin, std::size_t end) const {
-            return slice(_data + begin, _data + end);
+            return ArrayView<const T>(*this).slice(begin, end);
         }
 
         /**
@@ -455,11 +427,25 @@ class Array {
         }
         /** @overload */
         template<std::size_t size> StaticArrayView<size, T> slice(std::size_t begin) {
-            return slice<size>(_data + begin);
+            return ArrayView<T>(*this).template slice<size>(begin);
         }
         /** @overload */
         template<std::size_t size> StaticArrayView<size, const T> slice(std::size_t begin) const {
-            return slice<size>(_data + begin);
+            return ArrayView<const T>(*this).template slice<size>(begin);
+        }
+
+        /**
+         * @brief Fixed-size array slice
+         *
+         * At compile time expects that @cpp begin < end_ @ce, at runtime that
+         * @p end_ is not larger than @ref size().
+         */
+        template<std::size_t begin_, std::size_t end_> StaticArrayView<end_ - begin_, T> slice() {
+            return ArrayView<T>(*this).template slice<begin_, end_>();
+        }
+        /** @overload */
+        template<std::size_t begin_, std::size_t end_> StaticArrayView<end_ - begin_, const T> slice() const {
+            return ArrayView<const T>(*this).template slice<begin_, end_>();
         }
 
         /**
@@ -474,8 +460,23 @@ class Array {
         ArrayView<const T> prefix(const T* end) const {
             return ArrayView<const T>(*this).prefix(end);
         }
-        ArrayView<T> prefix(std::size_t end) { return prefix(_data + end); } /**< @overload */
-        ArrayView<const T> prefix(std::size_t end) const { return prefix(_data + end); } /**< @overload */
+        /** @overload */
+        ArrayView<T> prefix(std::size_t end) {
+            return ArrayView<T>(*this).prefix(end);
+        }
+        /** @overload */
+        ArrayView<const T> prefix(std::size_t end) const {
+            return ArrayView<const T>(*this).prefix(end);
+        }
+
+        /** @overload */
+        template<std::size_t viewSize> StaticArrayView<viewSize, T> prefix() {
+            return ArrayView<T>(*this).template prefix<viewSize>();
+        }
+        /** @overload */
+        template<std::size_t viewSize> StaticArrayView<viewSize, const T> prefix() const {
+            return ArrayView<const T>(*this).template prefix<viewSize>();
+        }
 
         /**
          * @brief Array suffix
@@ -489,8 +490,27 @@ class Array {
         ArrayView<const T> suffix(const T* begin) const {
             return ArrayView<const T>(*this).suffix(begin);
         }
-        ArrayView<T> suffix(std::size_t begin) { return suffix(_data + begin); } /**< @overload */
-        ArrayView<const T> suffix(std::size_t begin) const { return suffix(_data + begin); } /**< @overload */
+        /** @overload */
+        ArrayView<T> suffix(std::size_t begin) {
+            return ArrayView<T>(*this).suffix(begin);
+        }
+        /** @overload */
+        ArrayView<const T> suffix(std::size_t begin) const {
+            return ArrayView<const T>(*this).suffix(begin);
+        }
+
+        /**
+         * @brief Array prefix except the last @p count items
+         *
+         * Equivalent to @ref ArrayView::except().
+         */
+        ArrayView<T> except(std::size_t count) {
+            return ArrayView<T>(*this).except(count);
+        }
+        /** @overload */
+        ArrayView<const T> except(std::size_t count) const {
+            return ArrayView<const T>(*this).except(count);
+        }
 
         /**
          * @brief Release data storage
@@ -509,15 +529,10 @@ class Array {
 /** @relatesalso ArrayView
 @brief Make view on @ref Array
 
-Convenience alternative to calling @ref Array::operator ArrayView<U>()
-explicitly. The following two lines are equivalent:
+Convenience alternative to converting to an @ref ArrayView explicitly. The
+following two lines are equivalent:
 
-@code{.cpp}
-Containers::Array<std::uint32_t> data;
-
-Containers::ArrayView<std::uint32_t> a{data};
-auto b = Containers::arrayView(data);
-@endcode
+@snippet Containers.cpp Array-arrayView
 */
 template<class T, class D> inline ArrayView<T> arrayView(Array<T, D>& array) {
     return ArrayView<T>{array};
@@ -526,15 +541,10 @@ template<class T, class D> inline ArrayView<T> arrayView(Array<T, D>& array) {
 /** @relatesalso ArrayView
 @brief Make view on const @ref Array
 
-Convenience alternative to calling @ref Array::operator ArrayView<U>()
-explicitly. The following two lines are equivalent:
+Convenience alternative to converting to an @ref ArrayView explicitly. The
+following two lines are equivalent:
 
-@code{.cpp}
-const Containers::Array<std::uint32_t> data;
-
-Containers::ArrayView<const std::uint32_t> a{data};
-auto b = Containers::arrayView(data);
-@endcode
+@snippet Containers.cpp Array-arrayView-const
 */
 template<class T, class D> inline ArrayView<const T> arrayView(const Array<T, D>& array) {
     return ArrayView<const T>{array};
@@ -563,15 +573,6 @@ template<class T> std::size_t arraySize(const Array<T>& view) {
     return view.size();
 }
 
-#ifdef CORRADE_BUILD_DEPRECATED
-/** @brief @copybrief ArrayView
- * @deprecated Use @ref ArrayView.h and @ref ArrayView instead.
- */
-#ifndef CORRADE_MSVC2015_COMPATIBILITY /* Multiple definitions still broken */
-template<class T> using ArrayReference CORRADE_DEPRECATED_ALIAS("use ArrayView.h and ArrayView instead") = ArrayView<T>;
-#endif
-#endif
-
 template<class T, class D> inline Array<T, D>::Array(Array<T, D>&& other) noexcept: _data{other._data}, _size{other._size}, _deleter{other._deleter} {
     other._data = nullptr;
     other._size = 0;
@@ -595,6 +596,24 @@ template<class T, class D> inline Array<T, D>& Array<T, D>::operator=(Array<T, D
     return *this;
 }
 
+template<class T, class D> const T& Array<T, D>::front() const {
+    CORRADE_ASSERT(_size, "Containers::Array::front(): array is empty", _data[0]);
+    return _data[0];
+}
+
+template<class T, class D> const T& Array<T, D>::back() const {
+    CORRADE_ASSERT(_size, "Containers::Array::back(): array is empty", _data[_size - 1]);
+    return _data[_size - 1];
+}
+
+template<class T, class D> T& Array<T, D>::front() {
+    return const_cast<T&>(static_cast<const Array<T, D>&>(*this).front());
+}
+
+template<class T, class D> T& Array<T, D>::back() {
+    return const_cast<T&>(static_cast<const Array<T, D>&>(*this).back());
+}
+
 template<class T, class D> inline T* Array<T, D>::release() {
     /** @todo I need `std::exchange` NOW. */
     T* const data = _data;
@@ -603,6 +622,35 @@ template<class T, class D> inline T* Array<T, D>::release() {
     return data;
 }
 
+namespace Implementation {
+
+/* Array to ArrayView in order to have implicit conversion for StridedArrayView
+   without needing to introduce a header dependency. The SFINAE needs to be
+   here in order to ensure proper behavior with function overloads taking more
+   than one type of (Strided)ArrayView. */
+template<class U, class T, class D> struct ArrayViewConverter<U, Array<T, D>> {
+    template<class V = U> constexpr static typename std::enable_if<std::is_convertible<T*, V*>::value, ArrayView<U>>::type from(Array<T, D>& other) {
+        static_assert(sizeof(T) == sizeof(U), "types are not compatible");
+        return {&other[0], other.size()};
+    }
+};
+template<class U, class T, class D> struct ArrayViewConverter<const U, Array<T, D>> {
+    template<class V = U> constexpr static typename std::enable_if<std::is_convertible<T*, V*>::value, ArrayView<const U>>::type from(const Array<T, D>& other) {
+        static_assert(sizeof(T) == sizeof(U), "types are not compatible");
+        return {&other[0], other.size()};
+    }
+};
+template<class U, class T, class D> struct ArrayViewConverter<const U, Array<const T, D>> {
+    template<class V = U> constexpr static typename std::enable_if<std::is_convertible<T*, V*>::value, ArrayView<const U>>::type from(const Array<const T, D>& other) {
+        static_assert(sizeof(T) == sizeof(U), "types are not compatible");
+        return {&other[0], other.size()};
+    }
+};
+template<class T, class D> struct ErasedArrayViewConverter<Array<T, D>>: ArrayViewConverter<T, Array<T, D>> {};
+template<class T, class D> struct ErasedArrayViewConverter<const Array<T, D>>: ArrayViewConverter<const T, Array<T, D>> {};
+
+}
+
 }}
 
 #endif
