@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
  * @brief Class @ref Corrade::TestSuite::Compare::Less, @ref Corrade::TestSuite::Compare::LessOrEqual, @ref Corrade::TestSuite::Compare::GreaterOrEqual, @ref Corrade::TestSuite::Compare::Greater
  */
 
+#include "Corrade/TestSuite/Tester.h"
 #include "Corrade/TestSuite/TestSuite.h"
 #include "Corrade/Utility/Debug.h"
 
@@ -95,10 +96,10 @@ template<class T> class Greater {};
 /**
 @brief Pseudo-type for verifying that value is in given bounds
 
-Prints both values if @cpp actual < expected - epsilon @ce or
-@cpp expected + epsilon < actual @ce. Compared type needs to implement at least
-an @cpp operator-() @ce, @cpp operator+() @ce and @cpp operator<() @ce. Example
-usage:
+Prints both values if
+@cpp !(actual >= expected - epsilon && expected + epsilon >= actual) @ce.
+Compared type needs to implement at least an @cpp operator-() @ce,
+@cpp operator+() @ce and @cpp operator>=() @ce. Example usage:
 
 @snippet TestSuite.cpp Compare-Around
 
@@ -135,16 +136,17 @@ template<class T> inline Around<T> around(T epsilon) { return Around<T>{epsilon}
 #ifndef DOXYGEN_GENERATING_OUTPUT
 template<class T> class Comparator<Compare::Less<T>> {
     public:
-        bool operator()(const T& actual, const T& expected) {
+        ComparisonStatusFlags operator()(const T& actual, const T& expected) {
             _actualValue = &actual;
             _expectedValue = &expected;
-            return bool(*_actualValue < *_expectedValue);
+            return *_actualValue < *_expectedValue ?
+                ComparisonStatusFlags{} : ComparisonStatusFlag::Failed;
         }
 
-        void printErrorMessage(Utility::Error& e, const std::string& actual, const std::string& expected) const {
-            e << "Value" << actual << "is not less than" << expected
-              << Utility::Debug::nospace << ", actual is" << *_actualValue
-              << "but expected <" << *_expectedValue;
+        void printMessage(ComparisonStatusFlags, Utility::Debug& out, const char* actual, const char* expected) const {
+            out << "Value" << actual << "is not less than" << expected
+                << Utility::Debug::nospace << ", actual is" << *_actualValue
+                << "but expected <" << *_expectedValue;
         }
 
     private:
@@ -154,16 +156,17 @@ template<class T> class Comparator<Compare::Less<T>> {
 
 template<class T> class Comparator<Compare::LessOrEqual<T>> {
     public:
-        bool operator()(const T& actual, const T& expected) {
+        ComparisonStatusFlags operator()(const T& actual, const T& expected) {
             _actualValue = &actual;
             _expectedValue = &expected;
-            return bool(*_actualValue <= *_expectedValue);
+            return *_actualValue <= *_expectedValue ?
+                ComparisonStatusFlags{} : ComparisonStatusFlag::Failed;
         }
 
-        void printErrorMessage(Utility::Error& e, const std::string& actual, const std::string& expected) const {
-            e << "Value" << actual << "is not less than or equal to" << expected
-              << Utility::Debug::nospace << ", actual is" << *_actualValue
-              << "but expected <=" << *_expectedValue;
+        void printMessage(ComparisonStatusFlags, Utility::Debug& out, const char* actual, const char* expected) const {
+            out << "Value" << actual << "is not less than or equal to" << expected
+                << Utility::Debug::nospace << ", actual is" << *_actualValue
+                << "but expected <=" << *_expectedValue;
         }
 
     private:
@@ -173,16 +176,17 @@ template<class T> class Comparator<Compare::LessOrEqual<T>> {
 
 template<class T> class Comparator<Compare::GreaterOrEqual<T>> {
     public:
-        bool operator()(const T& actual, const T& expected) {
+        ComparisonStatusFlags operator()(const T& actual, const T& expected) {
             _actualValue = &actual;
             _expectedValue = &expected;
-            return bool(*_actualValue >= *_expectedValue);
+            return *_actualValue >= *_expectedValue ?
+                ComparisonStatusFlags{} : ComparisonStatusFlag::Failed;
         }
 
-        void printErrorMessage(Utility::Error& e, const std::string& actual, const std::string& expected) const {
-            e << "Value" << actual << "is not greater than or equal to" << expected
-              << Utility::Debug::nospace << ", actual is" << *_actualValue
-              << "but expected >=" << *_expectedValue;
+        void printMessage(ComparisonStatusFlags, Utility::Debug& out, const char* actual, const char* expected) const {
+            out << "Value" << actual << "is not greater than or equal to" << expected
+                << Utility::Debug::nospace << ", actual is" << *_actualValue
+                << "but expected >=" << *_expectedValue;
         }
 
     private:
@@ -192,16 +196,17 @@ template<class T> class Comparator<Compare::GreaterOrEqual<T>> {
 
 template<class T> class Comparator<Compare::Greater<T>> {
     public:
-        bool operator()(const T& actual, const T& expected) {
+        ComparisonStatusFlags operator()(const T& actual, const T& expected) {
             _actualValue = &actual;
             _expectedValue = &expected;
-            return bool(*_actualValue > *_expectedValue);
+            return *_actualValue > *_expectedValue ?
+                ComparisonStatusFlags{} : ComparisonStatusFlag::Failed;
         }
 
-        void printErrorMessage(Utility::Error& e, const std::string& actual, const std::string& expected) const {
-            e << "Value" << actual << "is not greater than" << expected
-              << Utility::Debug::nospace << ", actual is" << *_actualValue
-              << "but expected >" << *_expectedValue;
+        void printMessage(ComparisonStatusFlags, Utility::Debug& out, const char* actual, const char* expected) const {
+            out << "Value" << actual << "is not greater than" << expected
+                << Utility::Debug::nospace << ", actual is" << *_actualValue
+                << "but expected >" << *_expectedValue;
         }
 
     private:
@@ -211,23 +216,23 @@ template<class T> class Comparator<Compare::Greater<T>> {
 
 template<class T> class Comparator<Compare::Around<T>> {
     public:
-        /* GCC 4.7 and 4.9 (Travis CI Ubuntu and Travis CI Android) complain
-           about too many intiializers for _epsilon{epsilon} in the test that
-           uses an empty Bar structure. So can't use {} here. */
+        /* Has to be () and not {} otherwise GCC 4.8 complains that "too many
+           initializers for Bar" in NumericTest.cpp: */
         explicit Comparator(T epsilon): _epsilon(epsilon) {}
 
-        bool operator()(const T& actual, const T& expected) {
+        ComparisonStatusFlags operator()(const T& actual, const T& expected) {
             _actualValue = &actual;
             _expectedValue = &expected;
-            return !(*_actualValue < *_expectedValue - _epsilon ||
-                     *_expectedValue + _epsilon < *_actualValue);
+            return *_actualValue >= *_expectedValue - _epsilon &&
+                   *_expectedValue + _epsilon >= *_actualValue ?
+                ComparisonStatusFlags{} : ComparisonStatusFlag::Failed;
         }
 
-        void printErrorMessage(Utility::Error& e, const std::string& actual, const std::string& expected) const {
-            e << "Value" << actual << "is not around" << expected
-              << Utility::Debug::nospace << ", actual is" << *_actualValue
-              << "but" << *_expectedValue - _epsilon << "<= expected <="
-              << *_expectedValue + _epsilon;
+        void printMessage(ComparisonStatusFlags, Utility::Debug& out, const char* actual, const char* expected) const {
+            out << "Value" << actual << "is not around" << expected
+                << Utility::Debug::nospace << ", actual is" << *_actualValue
+                << "but" << *_expectedValue - _epsilon << "<= expected <="
+                << *_expectedValue + _epsilon;
         }
 
     private:
